<?php

/**
 * Blesta Reseller WHMCS Module
 *
 * Works with the Blesta Reseller API v2. Automates provisioning and managing reseller licenses
 * through your WHMCS installation.
 *
 * @author Jay Shah <jay@licensepal.com>
 * @version 1.0
 */

function blesta_ConfigOptions() {

    // Required configuration options
    $configarray = array(
        'Reseller Username' => array(
            'Type' => 'text',
            'Description' => 'Your Blesta account email or user',
            'Size' => '15'
        ),
        'Reseller Password' => array(
            'Type' => 'password',
            'Description' => 'Your Blesta account password',
            'Size' => '15'
        ),
        'License ID' => array(
            'Type' => 'text',
            'Description' => 'Acquire from Blesta (Usually 370 for monthly licenses and 364 for VAR and Tiered VAR)',
            'Size' => '5',
            'Default' => '370'
        ),
        'Test Mode'  => array(
            'Type' => 'yesno',
            'Description' => 'Enable only for debugging'
        )
    );

    return $configarray;

}

function blesta_CreateAccount($params) {

    $vars = array();

    // Set up the data for the action
    $vars['vars'] = array(
        'pricing_id' => (int) $params['configoption3']
    );

    // Add the license
    $result = blesta_MakeRequest('addlicense', $vars, $params['configoption1'], $params['configoption2'], $params['configoption4']);

    logModuleCall(
        'blesta',
        'addlicense',
        [$vars, $params['configoption1'], $params['configoption2'], $params['configoption4']],
        $result
    );

    if ($result['success']) {
        if (!empty($params['addonId'])) {
            // Update the custom license field with the new license key, if exists
            $table = 'tblcustomfields';
            $fields = "id,type,fieldname";
            $where = array(
                'fieldname' => 'License',
                'type' => 'addon',
                'relid' => blesta_EscapeString($params['packageid'])
            );
            $query = select_query($table, $fields, $where);
            $data = $query->fetch();
            
            if (!empty($data['id'])) {
                try {
                    $table = 'tblcustomfieldsvalues';
                    $values = array(
                        'fieldid' => blesta_EscapeString($data['id']),
                        'relid' => blesta_EscapeString($params['addonId']),
                        'value' => blesta_EscapeString($result['response'])
                    );
                    $newid = insert_query($table, $values);
                } catch (Throwable $e) {
                    // Nothing to do
                }
            }

            // Update the name field with the new license key
            $table = 'tblhostingaddons';
            $update = array('name' => blesta_EscapeString($result['response']));
            $where = array('id' => blesta_EscapeString($params['addonId']));
        } else {
            // Update the domain field with the new license key
            $table = 'tblhosting';
            $update = array('domain' => blesta_EscapeString($result['response']));
            $where = array('id' => blesta_EscapeString($params['serviceid']));
        }

        update_query($table, $update, $where);

        return 'success';
    }

    // The add failed, return the error
    return $result['message'];

}

function blesta_TerminateAccount($params) {

    $vars = array();

    // Get addon attributes
    if (!empty($params['addonId'])) {
        $attributes = blesta_GetAddonProperty($params['model'], 'attributes');

        if (!empty($attributes['name'])) {
            $params['domain'] = $attributes['name'];
        }
    }

    // Set up the data for the action
    $vars['vars'] = array(
        'license' => $params['domain']
    );

    // Terminate the license
    $result = blesta_MakeRequest('cancellicense', $vars, $params['configoption1'], $params['configoption2'], $params['configoption4']);

    logModuleCall(
        'blesta',
        'cancellicense',
        [$vars, $params['configoption1'], $params['configoption2'], $params['configoption4']],
        $result
    );

    if ($result['success']) {
        return 'success';
    }

    // The terminate failed, return the error
    return $result['message'];

}

function blesta_SuspendAccount($params) {

    $vars = array();

    // Get addon attributes
    if (!empty($params['addonId'])) {
        $attributes = blesta_GetAddonProperty($params['model'], 'attributes');

        if (!empty($attributes['name'])) {
            $params['domain'] = $attributes['name'];
        }
    }

    // Set up the data for the action
    $vars['vars'] = array(
        'license' => $params['domain']
    );

    // Suspend the license
    $result = blesta_MakeRequest('suspendlicense', $vars, $params['configoption1'], $params['configoption2'], $params['configoption4']);

    logModuleCall(
        'blesta',
        'suspendlicense',
        [$vars, $params['configoption1'], $params['configoption2'], $params['configoption4']],
        $result
    );

    if ($result['success']) {
        return 'success';
    }

    // The suspend failed, return the error
    return $result['message'];

}

function blesta_UnsuspendAccount($params) {

    $vars = array();

    // Get addon attributes
    if (!empty($params['addonId'])) {
        $attributes = blesta_GetAddonProperty($params['model'], 'attributes');

        if (!empty($attributes['name'])) {
            $params['domain'] = $attributes['name'];
        }
    }

    // Set up the data for the action
    $vars['vars'] = array(
        'license' => $params['domain']
    );

    // Unsuspend the license
    $result = blesta_MakeRequest('unsuspendlicense', $vars, $params['configoption1'], $params['configoption2'], $params['configoption4']);

    logModuleCall(
        'blesta',
        'unsuspendlicense',
        [$vars, $params['configoption1'], $params['configoption2'], $params['configoption4']],
        $result
    );

    if ($result['success']) {
        return 'success';
    }

    // The unsuspend failed, return the error
    return $result['message'];

}

function blesta_ClientArea($params) {

    $code = "<form action='clientarea.php' style='display: inline-block'>"
        . "<input type='hidden' name='action' value='productdetails' />"
        . "<input type='hidden' name='id' value='" . $params['serviceid'] . "' />"
        . "<input type='hidden' name='modop' value='custom' />"
        . "<input type='hidden' name='a' value='ReissueLicense' />"
        . "<input class='btn' type='submit' value='Reissue License' />"
        . "</form>"
        . " &nbsp; "
        . "<form action='https://account.blesta.com/client/plugin/download_manager/client_main/' style='display: inline-block'>"
        . "<input class='btn' type='submit' value='Download Section' />"
        . "</form>";

    return $code;

}

function blesta_ReissueLicense($params) {

    $vars = array();

    // Set up the data for the action
    $vars['vars'] = array(
        'license' => $params['domain'],
        'reissue_status' => 'reissue'
    );

    // Reissue the license
    $result = blesta_MakeRequest('update', $vars, $params['configoption1'], $params['configoption2'], $params['configoption4']);

    if ($result['success']) {
        return 'success';
    }

    // The reissue failed, return the error
    return $result['message'];

}

function blesta_ClientAreaCustomButtonArray() {

    // Add a reissue license option
    $buttonarray = array(
        'Reissue License' => 'ReissueLicense'
    );

    return $buttonarray;

}

function blesta_AdminCustomButtonArray() {

    // Add a reissue license option
    $buttonarray = array(
        'Reissue License' => 'ReissueLicense'
    );

    return $buttonarray;

}

function blesta_MakeRequest($action, $vars, $username, $password, $test_mode) {

    // API URL
    $url = 'https://account.blesta.com/plugin/blesta_reseller/v2/index/' . $action . '.json';

    // Enable test mode
    $vars['vars']['test_mode'] = ($test_mode == "on") ? 'true' : 'false';

    // Process curl request to perform this action
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($vars));
    curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
    curl_setopt($ch, CURLOPT_USERPWD, $username . ":" . $password);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

    $response = curl_exec($ch);
    $response_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

    // Decode the response
    $result = json_decode($response, true);

    // Add the HTTP code to the response
    $result['success'] = ($response_code == 200) ? true : false;

    // Log result
    logModuleCall('blesta', $action, $vars, $result);

    return $result;

}

function blesta_MetaData() {

    return array(
        'DisplayName' => 'Blesta',
        'APIVersion' => '2.0',
        'RequiresServer' => false
    );

}

function blesta_EscapeString($value) {

    $search = array("\\",  "\x00", "\n",  "\r",  "'",  '"', "\x1a");
    $replace = array("\\\\","\\0","\\n", "\\r", "\'", '\"', "\\Z");

    return str_replace($search, $replace, $value);

}

function blesta_GetAddonProperty($addon, $property) {

    $reflection = new ReflectionClass($addon);
    $property = $reflection->getProperty($property);
    $property->setAccessible(true);

    return $property->getValue($addon);

}
